import shutil
import importlib.util
import os
import tarfile
import subprocess as sp

def sync_policies():
    package_name = "anchorestig"
    spec = importlib.util.find_spec(package_name)
    package_root_directory = os.path.dirname(spec.origin)
    try:
        sp.run(["docker", "run", "--rm", "-d", "--name=policies", "anchore/combined-stig:latest", "sleep", "1000"])
        sp.run(["docker", "cp", "policies:policies/.", f"{package_root_directory}/policies"])
        sp.run(["docker", "kill", "policies"])
        directories = [d for d in os.listdir(f'{package_root_directory}/policies') if os.path.isdir(os.path.join(f'{package_root_directory}/policies', d))]
        print("\nAvailable profiles are:")
        for directory in directories:
            if directory != "input_templates":
                print(directory)
        print("\n")
    except Exception:
        print("Unauthorized to pull Anchore policies. Please login with the provided Docker credentials and try again.")
        exit()

def sync_profiles_from_tar(tar_path):
    package_name = "anchorestig"
    spec = importlib.util.find_spec(package_name)
    package_root_directory = os.path.dirname(spec.origin)
    try:
        file = tarfile.open(tar_path)
        file.extractall(f"{package_root_directory}")
        directories = [d for d in os.listdir(f'{package_root_directory}/policies') if os.path.isdir(os.path.join(f'{package_root_directory}/policies', d))]
        print("\nAvailable profiles are:")
        for directory in directories:
            if directory != "input_templates":
                print(directory)
        print("\n")
    except Exception:
        print("Unable to extract tarred policies. Please try again.")
        exit()

def check_saf_installed():
    # Check if 'saf' binary is available in PATH
    saf_path = shutil.which('saf')
    
    if saf_path:
        print(f"'saf' is installed at: {saf_path}")
    else:
        print("'saf' is not installed. If running this tool in a disconnected environment, please cancel this run with CTRL+C and manually install before proceeding.")
        
        # Prompt user to install via Homebrew or npm
        choice = input("Do you want to install 'saf' via Homebrew (1) or npm (2)? Enter 1 or 2: ")
        
        if choice == '1':
            install_command = "HOMEBREW_NO_AUTO_UPDATE=1 brew install mitre/saf/saf-cli"
        elif choice == '2':
            install_command = "npm install -g @mitre/saf"
        else:
            print("Invalid choice. Please enter 1 for Homebrew or 2 for npm.")
            return
        
        try:
            # Run the chosen installation command
            sp.run(install_command, shell=True, check=True)
            print(f"'saf' has been installed successfully using {install_command}.")
        except sp.CalledProcessError as e:
            print(f"Installation failed: {e}")

def check_cinc_installed():
    # Check if 'saf' binary is available in PATH
    cinc_path = shutil.which('cinc-auditor')
    
    if cinc_path:
        print(f"'CINC' is installed at: {cinc_path}")
    else:
        print("'CINC Auditor' is not installed. Please install this using `anchorestig provision --install`.")


import os
import subprocess
import importlib.util
import datetime
import shutil
import itertools
import threading
import time
import sys
import re
from pathlib import Path

def generate_input_file(profile_name):
    package_name = "anchorestig"
    spec = importlib.util.find_spec(package_name)
    package_root_directory = os.path.dirname(spec.origin)
    print("Retrieving input file template.")
    try:
        shutil.copyfile(f"{package_root_directory}/policies/input_templates/{profile_name}-template.yaml", f'{os.getcwd()}/{profile_name}-template.yaml')
    except:
        print("Input template not found.")
        exit(1)
    print(f"Input template: {profile_name}-template.yaml saved in the current directory")


import subprocess as sp
import logging
import importlib.util
import shutil
import os
from .inputs import *
import os.path
import boto3
import datetime
import itertools
import threading
import time
import sys
from botocore.exceptions import NoCredentialsError, ClientError

def upload_to_s3(bucket_name, file, account, image_digest, image_name):
    """
    Upload files from a specified directory to an AWS S3 bucket.

    :param bucket_name: Name of the S3 bucket.
    :param directory: Directory containing files to upload.
    """
    # Check if the directory exists
    if not os.path.exists(file):
        print(f"Error: output directory does not exist.")
        return

    # Initialize S3 client with custom credentials
    if "@" in image_name:
        image_name = image_name.split('@')[0] + "NOTAG"
    
    image_name = image_name.removeprefix('http://').removeprefix('https://')
    
    tag = image_name.split(':')[-1]
    registry = image_name.split('/', 1)[0]
    repository = image_name.split('/', 1)[-1].split(":")[0].replace("/", "-")

    s3 = boto3.client('s3')

    file_name = file.split("/")[-1]
    date_filename = add_date_prefix(file_name)

    try:
        # Walk through the directory and upload files
        file_path = f"anchore/{account}/{registry}/{repository}/{tag}/{image_digest}/{date_filename}"
        s3.upload_file(file, bucket_name, file_path)
    except NoCredentialsError:
        print("Error: AWS credentials not found.")
    except ClientError as e:
        print(f"Error: {e}")

def add_date_prefix(filename):
    date_prefix = str(datetime.datetime.now().timestamp()).replace(" ", "_").split(".")[0]
    split_filename = filename.rsplit(".", 1)
    return split_filename[0] + date_prefix + "." + split_filename[-1]

def completedStep():
    """ 
    Show progress of program in stdout 
    """
    global step, stepText
    if not step == totalSteps:
        step+=1
    print('\u2705 (' + str(step) + '/' + str(totalSteps) + ') -', stepText)

def failedStep(e):
        """ 
        Show error and exit if exception occurs 
        """
        print('\n\u274C Program failed. See details below:\n')
        print('Failed Step (' + str(step+1) + '/' + str(totalSteps) + '):', stepText, '\n')
        print(e,'\n')
        exit()

def scan():
    context = usecontext.replace(':',"--colon--")
    context = context.replace('/','--slash--')
    print(namespace)
    sp.run([
            "cinc-auditor","exec",controls,
            "-t","k8s-container://"+context+"/"+namespace+'/'+pod+'/'+container,
            "--input-file",profile,
            "--reporter=cli",
            "json:./output/"+outfile,
            "--log-level","debug"
            ],
            stdout=l, stderr=e, encoding='utf-8'
            )

def verifyOutput(outfile):
    """
    Verify that the output report has been successfully created and complete program execution
    """
    global step, stepText
    stepText = 'Output file verified.'
    print('  \u231B',stepText + '..', end="\r")
    try:
        if not os.path.isfile("./output/" + outfile):
            failedStep('Error generating output file. Please check the ./logs directory for more details.\n')
        with open("./output/" + outfile,"r") as f:
            length = len(f.readlines())
        if length < 1:
            failedStep('Error generating output file. Please check the ./logs directory for more details.\n')
        completedStep()
        print('\n The REM 2.0 STIG Analyzer process is complete. Please review your output here: ./output/' + outfile +'\n')
    except Exception as e: failedStep(e)
        
def saf_convert_output(outfile):
    conversion_tools = [ 'hdf2xccdf', 'hdf2ckl', 'hdf2csv']

    split_outfile = outfile.split('/')[-1].split('.json')[0]
    outfile_dir_exists = os.path.exists(f'./output/{split_outfile}')
    if not outfile_dir_exists: os.makedirs(f'./output/{split_outfile}')

    done = False
    def animate():
        print('Converting to additional output formats')
        for c in itertools.cycle(['|', '/', '-', '\\']):
            if done:
                break
            sys.stdout.write('\r' + c)
            sys.stdout.flush
            time.sleep(0.1)
        sys.stdout.write('\rDone converting.     ')
    t = threading.Thread(target=animate)
    t.start()

    for tool in conversion_tools:
        if tool == "hdf2xccdf":
            file_ending = ".xml"
        elif tool == "hdf2ckl":
            file_ending = ".ckl"
        else:
            file_ending = ".csv"
        conversion_cmd = f"saf convert {tool} -i ./output/{outfile} -o ./output/{split_outfile}/{split_outfile}{file_ending}" 
        try:
            # Run the chosen installation command
            sp.run(conversion_cmd, shell=True, check=True)
            # print(f"Successfully converted {outfile} using {conversion_cmd}.")
        except sp.CalledProcessError as e:
            print(f"Failed to convert {outfile} to {split_outfile}{file_ending}: {e}")
    done = True


def runtime_analysis(input_image, input_pod, input_container, input_namespace, input_usecontext, input_cluster, input_outfile, input_aws_s3_bucket_upload, input_account, input_image_digest, input_image_name):
    global step, stepText, e, totalSteps
    global image, pod, container, namespace, usecontext, cluster, outfile, controls, profile, aws_s3_bucket_upload, account, l, e

    # Set logger config and variables
    step = 0
    stepText = 'Setup complete. Running STIG Scan...'
    totalSteps = 3

    try:
        """
        Save input params as variables
        """
        logging.basicConfig(level=logging.INFO)
        log = logging.getLogger("REM 2.0")
        l = open('./logs/log.txt','w+')
        e = open('./logs/err.txt','w+')
        image = str(input_image).strip()
        pod = input_pod.strip()
        container = str(input_container).strip()
        namespace = input_namespace.strip()
        usecontext = input_usecontext.strip()
        cluster = input_cluster.strip()
        outfile = input_outfile.strip()
        if input_aws_s3_bucket_upload != 'skip_upload':
            aws_s3_bucket_upload = input_aws_s3_bucket_upload.strip()
            account = input_account.strip()
        image_digest = input_image_digest
        image_name = input_image_name
        outfile = input_outfile
        out_ext = outfile[outfile.index('.')+1:len(outfile)]
        outdir_exists = os.path.exists(f'./output/{outfile.split("/")[0]}')
        if not outdir_exists: os.makedirs(f'./output/{outfile.rsplit("/")[0]}')
        out_dir='./'

        package_name = "anchorestig"
        spec = importlib.util.find_spec(package_name)
        package_root_directory = os.path.dirname(spec.origin)

        if image == 'ubuntu-20.04': 
            image = 'ubuntu2004'
            policy_dir = f'{package_root_directory}/policies/ubuntu2004/'
        elif image == 'ubi8': policy_dir = f'{package_root_directory}/policies/ubi8/'
        elif image == 'postgres9': policy_dir = f'{package_root_directory}/policies/postgres9/'
        elif image == 'apache-tomcat9': policy_dir = f'{package_root_directory}/policies/apache-tomcat9/'
        elif image == 'crunchy-postgresql': policy_dir = f'{package_root_directory}/policies/crunchy-postgresql/'
        elif image == 'jboss': policy_dir = f'{package_root_directory}/policies/jboss/'
        elif image == 'jre7': policy_dir = f'{package_root_directory}/policies/jre7/'
        elif image == 'mongodb': policy_dir = f'{package_root_directory}/policies/mongodb/'
        elif image == 'nginx': policy_dir = f'{package_root_directory}/policies/nginx/'
        elif image == 'ubi9': policy_dir = f'{package_root_directory}/policies/ubi9/'
        elif image == 'ubuntu2204': policy_dir = f'{package_root_directory}/policies/ubuntu2204/'
        else:
            failedStep('Configuration and variables failed. Please ensure that you\'re using the correct input parameter values.')

        profile = policy_dir + 'profile.yaml'
        controls = policy_dir + 'anchore-'+image+'-disa-stig-1.0.0.tar.gz'
        proc = sp.Popen(["kubectl","--context",usecontext,"--namespace",namespace,"get","pods","-o","jsonpath='{..containers[*].name}'"], stdout=sp.PIPE)
        output = proc.stdout.read().decode("utf-8").strip().replace("'","")
        containerList = output.split(' ')

        completedStep()
    except Exception as e: failedStep(e)

    try:
        scan()
        # Check for errors
        e.close()
        if os.stat("./logs/err.txt").st_size != 0: 
            raise Exception("There was an error with the scan. Please review the ./logs/err.txt file for more information.")
        else:
            e = open('./logs/err.txt','w+')
            stepText = 'STIG scan complete. Verifying output file...'
            if input_aws_s3_bucket_upload != 'skip_upload':
                outfile_raw_dir = f"{outfile.rsplit('.', 1)[0]}"
                output_file_dir = f"./output/{outfile.split('/')[0]}"
                for file in os.listdir(output_file_dir):
                    upload_to_s3(aws_s3_bucket_upload, f'./{output_file_dir}/{file}', account, image_digest, image_name)
            completedStep()
    except Exception as e: failedStep(e)

    verifyOutput(outfile)


import os
import subprocess
import importlib.util
import datetime
import shutil
import itertools
import threading
import time
import sys
import re
from pathlib import Path

def determine_profile(profile):
    package_name = "anchorestig"
    spec = importlib.util.find_spec(package_name)
    package_root_directory = os.path.dirname(spec.origin)

    if profile == "ubi8" or profile == "ubuntu2004" or profile == "ubi9" or profile == "ubuntu2204":
        policy_path = f"{package_root_directory}/policies/{profile}/anchore-{profile}-disa-stig-1.0.0.tar.gz"
    else:
        policy_path = profile
    return policy_path

def saf_convert_output(outfile):
    conversion_tools = [ 'hdf2xccdf', 'hdf2ckl', 'hdf2csv']

    split_outfile = outfile.split('/')[-1].split('.json')[0]

    done = False
    def animate():
        print('Converting to additional output formats')
        for c in itertools.cycle(['|', '/', '-', '\\']):
            if done:
                break
            sys.stdout.write('\r' + c)
            sys.stdout.flush
            time.sleep(0.1)
        sys.stdout.write('\rDone converting.     ')
    t = threading.Thread(target=animate)
    t.start()

    for tool in conversion_tools:
        if tool == "hdf2xccdf":
            file_ending = ".xml"
        elif tool == "hdf2ckl":
            file_ending = ".ckl"
        else:
            file_ending = ".csv"
        conversion_cmd = f"saf convert {tool} -i {outfile} -o ./stig-results/{outfile.split('/')[2]}/{split_outfile.split('-output')[0]}/{split_outfile}{file_ending}" 
        # print(conversion_cmd)
        try:
            Path(f"./stig-results/{outfile.split('/')[2]}/{split_outfile.split('-output')[0]}").mkdir(parents=True, exist_ok=True)
            # Run the chosen installation command
            subprocess.run(conversion_cmd, shell=True, check=True)
        except subprocess.CalledProcessError as e:
            print(f"Failed to convert {outfile} to {split_outfile}{file_ending}: {e}")
    shutil.move(outfile, f"./stig-results/{outfile.split('/')[2]}/{split_outfile.split('-output')[0]}/")
    done = True

def run_stig(output_dir, policy_path, input_file, ssh_user, ssh_password, ssh_host, ssh_key_path, sanitized_usertime):
    
    try:
        if input_file == "default":
            if ssh_password == "usekey":
                response = subprocess.run(["cinc-auditor", "exec", policy_path, "-t", f"ssh://{ssh_user}@{ssh_host}", "-i", ssh_key_path, "--reporter=cli", f"json:./{output_dir}/{sanitized_usertime}-output.json"], stdout=subprocess.PIPE)
            else:
                response = subprocess.run(["cinc-auditor", "exec", policy_path, "-t", f"ssh://{ssh_user}:{ssh_password}@{ssh_host}", "--reporter=cli", f"json:./{output_dir}/{sanitized_usertime}-output.json"], stdout=subprocess.PIPE)
        else:
            if not os.path.isfile(input_file):
                print(f"Input file: {input_file} does not exist. Please Retry.")
                pass
            else:
                if ssh_password == "usekey":
                    response = subprocess.run(["cinc-auditor", "exec", policy_path, "-t", f"ssh://{ssh_user}@{ssh_host}", "-i", ssh_key_path, f"--input-file={input_file}", "--reporter=cli", f"json:./{output_dir}/{sanitized_usertime}-output.json"], stdout=subprocess.PIPE)
                else:
                    response = subprocess.run(["cinc-auditor", "exec", policy_path, "-t", f"ssh://{ssh_user}:{ssh_password}@{ssh_host}", f"--input-file={input_file}", "--reporter=cli", f"json:./{output_dir}/{sanitized_usertime}-output.json"], stdout=subprocess.PIPE)
    except Exception:
        print("Failed to run STIG")
        stop_container(container_id)
        exit()

# Execute profiles on the remote machine
def run_stig_over_ssh(profile, input_file, host, user, password, key):
    try:
        policy_path = determine_profile(profile)
        dir_name = host.replace("/", "-").replace(":", "-")
        os.makedirs(f"stig-results/{dir_name}", exist_ok=True)

        now = datetime.datetime.now()
        sanitized_usertime = f"{user}-{now}".replace(" ", "-").replace("/", "-").replace(":", "-")

        print("\n-------Run Parameters-------\n")
        print(f"Target Host: {host}")
        print(f"Profile: {profile}")
        print(f"User: {user}")
        print(f'Output File Path: ./stig-results/{dir_name}/{sanitized_usertime}-output.json\n')

        run_stig(f"stig-results/{dir_name}", policy_path, input_file, user, password, host, key, sanitized_usertime)
        saf_convert_output(f"./stig-results/{dir_name}/{sanitized_usertime}-output.json")

    except Exception as e:
        print(f"Error running profile {profile}: {e}")



import getopt, sys
import subprocess as sp
import json
import os

def selectFromDict(options, name):
    """ 
    Have the user select the container image that they are analyzing. Used in demo.main() to select
    and download the appropriate SSG policy. 
    """
    index = 0
    indexValidList = []
    print("\nPlease select an image profile (enter '1' for ubuntu:20.04, '2' for ubi8, '3' for postgre9, '4' for apache tomcat 9, '5' for crunchy postgresql, '6' for jboss, '7' for jre 7, '8' for mongodb, '9' for nginx, '10' for ubi9, or '11' for ubuntu:22.04):\n")
    for optionName in options:
        index = index + 1
        indexValidList.extend([options[optionName]])
        print(str(index) + '. ' + optionName)
    inputValid = False
    while not inputValid:
        inputRaw = input("\nImage Profile (enter 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, or 11): ")
        if inputRaw.isnumeric():
            inputNo = int(inputRaw) - 1
            if inputNo > -1 and inputNo < len(indexValidList):
                selected = indexValidList[inputNo]
                inputValid = True
                break
            else:
                print('\nNot a valid selection. Please select a valid ' + name)
        else:
            print('\nNot a valid selection. Please select a valid ' + name)
    return selected

def runtime_get_image_digest(pod, namespace, container):
    logdir_exists = os.path.exists('./logs')
    if not logdir_exists: os.makedirs('./logs')
    errout = open('./logs/err.txt','w+')
    image_name = sp.run(['kubectl', 'get', 'pod', pod, '-n', namespace, '-o', f'jsonpath=\"{{.status.containerStatuses[?(@.name==\'{container}\')].image}}\"'], stdout=sp.PIPE, stderr=errout, encoding='utf-8').stdout.strip().strip('"')
    image_digest = sp.run(['kubectl', 'get', 'pod', pod, '-n', namespace, '-o', f'jsonpath=\"{{.status.containerStatuses[?(@.name==\'{container}\')].imageID}}\"'], stdout=sp.PIPE, stderr=errout, encoding='utf-8').stdout.strip().strip('"').split("@")[-1]
    return image_digest, image_name

def get_runtime_cluster(usecontext):
    logdir_exists = os.path.exists('./logs')
    if not logdir_exists: os.makedirs('./logs')
    errout = open('./logs/err.txt','w+')
    try:
        proc = sp.run(['kubectl','config','view','-o',"jsonpath='{.contexts[?(@.name==\""+usecontext+"\")]}'"], stdout=sp.PIPE, stderr=errout, encoding='utf-8')
        contextdata = proc.stdout.strip().strip("'")
        contextdata = json.loads(contextdata)
        contextdata = contextdata['context']
        if "cluster" in contextdata: 
            cluster = contextdata['cluster']
            if cluster == '': 
                raise Exception("Cluster value is blank or not found in context config file")
            return cluster
        else: 
            raise Exception("Cluster not found within context. Please add a cluster to your context config and try again.")
        if "namespace" in contextdata: 
            print("\n**NOTE: A namespace has been found in the context you're using. Please note that you will still be required to provide a \
                namespace that will be used in the program execution if you have not already included one with CLI input.")

    except Exception as e:
        print('\n\u274C There was an error finding the cluster with the context provided. Please review your input, verify that the context exists, and try again.\n\n', e)
        exit()

def collect_inputs():
    print('\n -- REM 2.0: Container Runtime Automated STIG Analyzer --')

    # Set variables used to collect input parameters
    clear = '\x1b[2K\n'
    args = sys.argv[1:]
    shortopts = 'i:p:c:o:n:u:h'
    longopts = ['image=','pod=','container=','outfile=','namespace=','usecontext=','help']
    options = {}
    options['Ubuntu 20.04'] = 'ubuntu-20.04'
    options['Universal Base Image 8 (ubi8)'] = 'ubi8'
    options['Postgres 9'] = 'postgres9'
    options['Apache Tomcat 9'] = 'apache-tomcat9'
    options['Crunchy PostgreSQL'] = 'crunchy-postgresql'
    options['JBOSS'] = 'jboss'
    options['JRE 7'] = 'jre7'
    options['MongoDB'] = 'mongodb'
    options['nginx'] = 'nginx'
    options['Ubuntu 22.04'] = 'ubuntu2204'
    options['Universal Base Image 9 (ubi9)'] = 'ubi9'
    helpflag = False
    container = False
    namespace = False
    usecontext = False
    podprinted = False
    imageprinted = False
    aws_s3_bucket_upload = False
    account = False
    cluster = ''
    logdir_exists = os.path.exists('./logs')
    if not logdir_exists: os.makedirs('./logs')
    errout = open('./logs/err.txt','w+')

    try:
        """ 
        Save input args to variables, or show Help text if the --help argument was input 
        """
        arguments, values = getopt.getopt(args, shortopts, longopts)
        for currentArgument, currentValue in arguments:
            if currentArgument in ("-h", "--help"):
                print('Please see README located at: https://github.com/bknfzr/stig\n')
                helpflag = True
                exit()
            if currentArgument in ("-i", "--image"):
                image = currentValue
            if currentArgument in ("-p", "--pod"):
                pod = currentValue
            if currentArgument in ("-c", "--container"):
                container = currentValue 
            if currentArgument in ("-o", "--outfile"):
                outfile = currentValue
            if currentArgument in ("-n", "--namespace"):
                namespace = currentValue
            if currentArgument in ("-u","--usecontext"):
                usecontext = currentValue
            if currentArgument in ("-b", "--aws-bucket"):
                aws_s3_bucket_upload = currentValue
            if currentArgument in ("-a", "--account"):
                account = currentValue
    except getopt.error as err:
        print ('\n\u274C The program failed, see the following error\n\n:' + str(err))

    if not helpflag:
        """ 
        Collect input variables for all remaining arguments 
        """
        try:
            contextExists = False
            while not contextExists:
                # Set usecontext variable
                if not usecontext:
                    usecontext = input("\nPlease provide the context to use (to use the current context, type 'current' or press ENTER): ")
                    if usecontext == '':
                        usecontext = 'current'
                if usecontext == 'current':
                    try:
                        print('fetching name of current context...', end='\r')
                        proc = sp.run(['kubectl','config','current-context'], stdout=sp.PIPE, stderr=errout, encoding='utf-8')
                        usecontext = proc.stdout.strip()
                    except Exception as e:
                        print('\n\u274C Error obtaining the current user context:\n\n', e)
                        exit()
                else:
                    try:
                        proc = sp.run(['kubectl','config','view','-o',"jsonpath='{.contexts[*].name}'"], stdout=sp.PIPE, stderr=errout, encoding='utf-8')
                        contexts = proc.stdout.strip()
                        contexts = contexts.strip("'").split()
                        if usecontext in contexts: 
                            contextExists = True
                        else: 
                            usecontext = False
                            print('\n\u274C The context provided was not found.')
                    except Exception as e:
                        print('\n\u274C There was an error with the provided context. Please review your input, verify that the context exists, and try again.\n\n', e)
                        exit()
            
            print(clear, '\nContext:',usecontext)

            # Get context info
            try:
                proc = sp.run(['kubectl','config','view','-o',"jsonpath='{.contexts[?(@.name==\""+usecontext+"\")]}'"], stdout=sp.PIPE, stderr=errout, encoding='utf-8')
                contextdata = proc.stdout.strip().strip("'")
                contextdata = json.loads(contextdata)
                contextdata = contextdata['context']
                if "cluster" in contextdata: 
                    cluster = contextdata['cluster']
                    if cluster == '': 
                        raise Exception("Cluster value is blank or not found in context config file")
                    print("\nCluster: ", cluster)
                else: 
                    raise Exception("Cluster not found within context. Please add a cluster to your context config and try again.")
                if "namespace" in contextdata: 
                    print("\n**NOTE: A namespace has been found in the context you're using. Please note that you will still be required to provide a \
    namespace that will be used in the program execution if you have not already included one with CLI input.")

            except Exception as e:
                print('\n\u274C There was an error finding the cluster with the context provided. Please review your input, verify that the context exists, and try again.\n\n', e)
                exit()

            # Set namespace variable
            if not namespace:
                namespace = input("\nPlease provide the namespace name (to use the default namespace, type 'default' or press ENTER): ")
                if namespace == '':
                    namespace = 'default'
            try:
                proc = sp.run(['kubectl','get','namespaces','--context', usecontext, '-o',"jsonpath='{.items[*].metadata.labels.kubernetes\.io/metadata\.name}'"], stdout=sp.PIPE, stderr=sp.PIPE, encoding='utf-8')
                namespaceerr = proc.stderr
                if namespaceerr:                
                    raise Exception("The provided namespace errored out.\n\n", namespaceerr)
                else:
                    namespaces = proc.stdout.strip().strip("'").split()
                    if namespace not in namespaces:
                        raise Exception("The namespace:'" + namespace + "' could not be found.\n")
                    print('\nNamespace:',namespace)
            except Exception as e:
                print('\n\u274C Error with the provided namespace: ' + namespace + '.\n\n', e)
                exit()

            # Set image variable
            if not 'image' in globals():
                image = selectFromDict(options, 'image')
                print('\nImage:',image)  
                imageprinted = True
            elif not imageprinted: print('\nImage:',image) 
            
            # Set pod variable
            if not 'pod' in globals():
                pod = input("\nPlease provide the pod name: ")
                print('Verifying that pod exists...', end='\r')
                print(clear + 'Pod:',pod)
                podprinted = True
            checkforpod = sp.run(['kubectl','get','pods',pod,'-n',namespace, '--context', usecontext], stdout=sp.DEVNULL, stderr=sp.PIPE, encoding='utf-8')
            poderr = checkforpod.stderr
            if poderr:
                raise Exception("The pod:'" + pod + "' could not be found. Please retry with a running pod.\n")
            elif not podprinted: print('\nPod:',pod)

            # Set container variable
            if not container:
                container = input("\nPlease provide the container name (to use the default container, type 'default' or press ENTER): ")
                if container == '':
                    container = 'default'
            if container == 'default':
                try:
                    print('fetching name of default container...', end='\r')
                    proc = sp.run(['kubectl','get','pods',pod,'-n',namespace, '--context', usecontext,'-o',"jsonpath='{.spec.containers[0].name}'"], stdout=sp.PIPE, stderr=sp.PIPE, encoding='utf-8')
                    containererr = proc.stderr
                    if containererr:
                        raise Exception(containererr)
                    container = str(proc.stdout).strip("'")
                    print(clear + 'Container:',container)
                except Exception as e:
                    print('\n\u274C Error obtaining the default container name for the Pod: ' + pod + ': \n\n', e)
                    exit()
            else:
                try:
                    print('fetching name of default container...', end='\r')
                    proc = sp.run(['kubectl','get','pods',pod,'-n',namespace, '--context', usecontext,'-o',"jsonpath='{.spec.containers[*].name}'"], stdout=sp.PIPE, stderr=sp.PIPE, encoding='utf-8')
                    containererr = proc.stderr
                    if containererr:                
                        raise Exception("The provided container name errored out.\n\n", containererr)
                    else:
                        containers = proc.stdout.strip().strip("'").split()
                        if container not in containers:
                            raise Exception("The container:'" + container + "' could not be found.\n")
                    print(clear + 'Container:',container)
                except Exception as e:
                    print('\n\u274C Error finding the container name: ' + container + ' provided for the Pod: ' + pod + ': \n\n', e)
                    exit()

            # Set output filename + extension variable
            if not 'outfile' in globals():
                outfile = input("\nPlease provide a json output filename, incl. name + ext (e.g. 'Results.json'): ")
                type = outfile.endswith('.json')
                while not type:
                    outfile = input("\nIncorrect file name/format. Please provide a json output filename, incl. name + ext (e.g. 'Results.json'): ")
                    type = outfile.endswith('.json')
            print('\n')

            if not 'aws_s3_bucket_upload' in globals():
                aws_s3_bucket_upload = input("\nPlease provide an S3 bucket to upload the output file to (to skip the upload press ENTER): ")
                if aws_s3_bucket_upload == '':
                    aws_s3_bucket_upload = 'skip_upload'
                    image_digest, image_name = runtime_get_image_digest(pod, namespace, container)
                if aws_s3_bucket_upload != 'skip_upload':
                    account = input("\nPlease provide an Anchore STIG UI account: ")
                    image_digest, image_name = runtime_get_image_digest(pod, namespace, container)
            print('\n')

            # Check for errors
            errout.close()
            if os.stat("./logs/err.txt").st_size != 0: 
                raise Exception("There was an error with the inputs provided. Please review the ./logs/err.txt file for more information.")
        except Exception as e:
            print('\n\n\u274C The following error has occurred:\n\n', e)
            exit()
        
        return image, pod, container, namespace, usecontext, cluster, outfile, aws_s3_bucket_upload, account, image_digest, image_name


import subprocess as sp
import sys
import logging
import importlib.util
import os
import os.path

def install_cinc(privileged):
    if privileged:
        cmd = ["curl", "-L", "https://omnitruck.cinc.sh/install.sh", "|", "sudo", "bash", "-s", "--", "-P", "cinc-auditor", "-v", "5.22.50"]
    else:
        cmd = ["curl", "-L", "https://omnitruck.cinc.sh/install.sh", "|", "bash", "-s", "--", "-P", "cinc-auditor", "-v", "5.22.50"]
    try:
        sp.Popen(cmd, stdout=sp.PIPE, stderr=sp.PIPE)
    except Exception:
        print("Failed to install CINC auditor. Please try again or manually install by running 'curl -L https://omnitruck.cinc.sh/install.sh | sudo bash -s -- -P cinc-auditor -v 5.22.50'")
        exit()

def install_train_plugin():
    package_name = "anchorestig"
    spec = importlib.util.find_spec(package_name)
    train_k8s_container_path = f"{os.path.dirname(spec.origin)}/train-k8s-container"
    print(train_k8s_container_path)
    cmd = ["cinc-auditor", "plugin", "install", train_k8s_container_path]
    try:
        sp.Popen(cmd, stdout=sys.stdout, stderr=sys.stderr)
    except Exception:
        print("Failed to install the train k8s container plugin.")
        exit()


import json
import os
import subprocess as sp

def validate_file(file_path):
    if not os.path.isfile(file_path):
        print("Error: File does not exist.")
        return False

    if not file_path.lower().endswith('.json'):
        print("Error: File does not have a .json extension.")
        return False

    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            json.load(file)
        return True
    except json.JSONDecodeError:
        print("Error: File contains invalid JSON.")
        return False
    except Exception as e:
        print(f"Error: {e}")
        return False

def create_output_rollup(output_file_path):
    review_command = ["saf", "view", "summary", "-i", output_file_path, "-r"]
    try:
        # Run the chosen installation command
        result = sp.run(review_command, capture_output=True, text=True)
        print(result.stdout)
    except sp.CalledProcessError as e:
        print(f"Failed to output document review: {e}")


import argparse
import os
import subprocess
import signal
import boto3
import importlib.util
import datetime
import itertools
import threading
import time
import sys
import re
import tarfile
from botocore.exceptions import NoCredentialsError, ClientError

def usage():
    print("Usage: anchorestig static <IMAGE> [-u registry_username] [-p registry_password] [-r registry_url] [-s] [-b AWS S3 Bucket name] [-a Anchore STIG UI Account] [-l STIG Profile nae or path] [-i Input File for STIG]")
    exit(1)

def stop_container(container_id):
    command = ['docker', 'kill', container_id]
    try:
        result = subprocess.run(command, capture_output=True, text=True)
    except Exception:
        print(f"Unable to stop sandbox container. Please manually stop the container")

def add_date_prefix(filename):
    date_prefix = str(datetime.datetime.now().timestamp()).replace(" ", "_").split(".")[0]
    split_filename = filename.rsplit(".", 1)
    return split_filename[0] + date_prefix + "." + split_filename[-1]

def upload_to_s3(bucket_name, file, account, image_digest, image_name):
    """
    Upload files from a specified directory to an AWS S3 bucket.

    :param bucket_name: Name of the S3 bucket.
    :param directory: Directory containing files to upload.
    """
    # Check if the directory exists
    if not os.path.exists(file):
        print(f"Error: output directory does not exist.")
        return

    # Initialize S3 client with custom credentials
    if "@" in image_name:
        image_name = image_name.split('@')[0] + "NOTAG"
    
    image_name = image_name.removeprefix('http://').removeprefix('https://')
    
    tag = image_name.split(':')[-1]
    registry = image_name.split('/', 1)[0]
    repository = image_name.split('/', 1)[-1].split(":")[0].replace("/", "-")

    s3 = boto3.client('s3')

    file_name = file.split("/")[-1]
    date_filename = add_date_prefix(file_name)

    try:
        # Walk through the directory and upload files
        file_path = f"anchore/{account}/{registry}/{repository}/{tag}/{image_digest}/{date_filename}"
        s3.upload_file(file, bucket_name, file_path)
    except NoCredentialsError:
        print("Error: AWS credentials not found.")
    except ClientError as e:
        print(f"Error: {e}")

def get_image_digest(image, username, password, url, insecure):
    try:
        if not username:
            pull_cmd = ["docker", "pull", image]
        else:
            subprocess.run(["docker", "--config", "./stig-docker-config", "login", url, "-u", username, "-p", password], check=True)
            pull_cmd = ["docker", "pull", image]

        # Try to pull the image
        pull_result = subprocess.run(pull_cmd, capture_output=True, text=True)
        if pull_result.returncode != 0:
            raise RuntimeError(f"Failed to pull image {image}: {pull_result.stderr.strip()}")

        # Try to inspect the image digest
        inspect_result = subprocess.run(["docker", "inspect", '--format={{index .RepoDigests 0}}', image], capture_output=True, text=True)
        if inspect_result.returncode != 0 or not inspect_result.stdout.strip():
            raise RuntimeError(f"Failed to inspect image {image}: {inspect_result.stderr.strip()}")

        # Extract and print the digest
        digest = inspect_result.stdout.strip().split('@')[-1].replace('"', '')
        print(digest)

        # Remove the image after use
        subprocess.run(["docker", "rmi", image], check=False)

        return digest

    except Exception as e:
        print(f"Error: {e}")
        return None

def determine_base_image(container_id):
    try:
        response = subprocess.run(["docker", "exec", "-it", container_id, "cat", "/etc/os-release"], capture_output=True, text=True)
        base_distro = response.stdout
        pattern = r"PRETTY_NAME=(.*?)(?:\n|$)"
        match = re.search(pattern, base_distro, re.DOTALL)
        base_distro = match.group(1).strip().lower()
        if "red hat enterprise linux 8" in base_distro:
            return "ubi8"
        elif "red hat enterprise linux 9" in base_distro:
            return "ubi9"
        elif "ubuntu 20" in base_distro:
            return "ubuntu2004"
        elif "ubuntu 22" in base_distro:
            return "ubuntu2204"
        else:
            return "indeterminate"
    except Exception:
        print("Failed to determine base distribution.")
        stop_container(container_id)
        exit()

def get_container_id(image):
    try:
        response = subprocess.run(["docker", "run", "-t", "-d", "--entrypoint=cat", image], capture_output=True, text=True)
        container_id = response.stdout.strip()
        return container_id
    except Exception:
        print("Failed to start target image. Please verify and retry.")
        exit()

def saf_convert_output(outfile):
    conversion_tools = [ 'hdf2xccdf', 'hdf2ckl', 'hdf2csv']

    split_outfile = outfile.split('/')[-1].split('.json')[0]

    done = False
    def animate():
        print('Converting to additional output formats')
        for c in itertools.cycle(['|', '/', '-', '\\']):
            if done:
                break
            sys.stdout.write('\r' + c)
            sys.stdout.flush
            time.sleep(0.1)
        sys.stdout.write('\rDone converting.     ')
    t = threading.Thread(target=animate)
    t.start()

    for tool in conversion_tools:
        if tool == "hdf2xccdf":
            file_ending = ".xml"
        elif tool == "hdf2ckl":
            file_ending = ".ckl"
        else:
            file_ending = ".csv"
        conversion_cmd = f"saf convert {tool} -i {outfile} -o ./stig-results/{split_outfile.split('-output')[0]}/{split_outfile}{file_ending}" 
        try:
            # Run the chosen installation command
            subprocess.run(conversion_cmd, shell=True, check=True)
        except subprocess.CalledProcessError as e:
            print(f"Failed to convert {outfile} to {split_outfile}{file_ending}: {e}")
    done = True

def run_stig(container_id, output_dir, image, policy_path, input_file):
    sanitized_image = image.replace("/", "-").replace(":", "-")
    try:
        if input_file == "default":
            response = subprocess.run(["cinc-auditor", "exec", policy_path, "-t", f"docker://{container_id}", "--reporter=cli", f"json:./{output_dir}/{sanitized_image}-output.json"], stdout=subprocess.PIPE)
        else:
            if not os.path.isfile(input_file):
                print(f"Input file: {input_file} does not exist. Please Retry.")
                stop_container(container_id)
                pass
            else:
                response = subprocess.run(["cinc-auditor", "exec", policy_path, "-t", f"docker://{container_id}", f"--input-file={input_file}", "--reporter=cli", f"json:./{output_dir}/{sanitized_image}-output.json"], stdout=subprocess.PIPE)
    except Exception:
        print("Failed to run STIG")
        stop_container(container_id)
        exit()

def static_analysis(username, password, url, insecure, image, aws_s3_bucket_upload, account, profile, input_file):

    if not image:
        usage()

    dir_name = image.replace("/", "-").replace(":", "-")
    os.makedirs(f"stig-results/{dir_name}", exist_ok=True)

    container_id = get_container_id(image)

    if profile == "auto":
        profile = determine_base_image(container_id)
    if profile == "indeterminate":
        print("No automatic profile available, please specify a profile to continue.")
        exit(1)

    package_name = "anchorestig"
    spec = importlib.util.find_spec(package_name)
    package_root_directory = os.path.dirname(spec.origin)

    if profile == "ubi8" or profile == "ubi9" or profile == "ubuntu2004" or profile == "ubuntu2204" or profile == "nginx" or profile == "crunchy-postgresql" or profile == "jboss" or profile == "jre7" or profile == "mongodb" or profile == "postgres9":
        policy_path = f"{package_root_directory}/policies/{profile}/anchore-{profile}-disa-stig-1.0.0.tar.gz"
    else:
        policy_path = profile

    print("\n-------Run Parameters-------\n")
    print(f"Target Image: {image}")
    print(f"Profile: {profile}")
    print(f"Input File: {input_file}")
    print(f'Output File Path: ./stig-results/{dir_name}/{image.replace("/", "-").replace(":", "-")}-output.json\n')

    run_stig(container_id, f"stig-results/{dir_name}", image, policy_path, input_file)

    stop_container(container_id)

    saf_convert_output(f'./stig-results/{dir_name}/{image.replace("/", "-").replace(":", "-")}-output.json')

               
    if aws_s3_bucket_upload:
        image_digest = get_image_digest(image, username, password, url, insecure)
        done = False
        def animate():
            print('Uploading results to S3.')
            for c in itertools.cycle(['|', '/', '-', '\\']):
                if done:
                    break
                sys.stdout.write('\r' + c)
                sys.stdout.flush
                time.sleep(0.1)
            sys.stdout.write('\rDone!     ')
        t = threading.Thread(target=animate)
        t.start()
        for file in os.listdir(f"{os.getcwd()}/stig-results/{dir_name}"):
            upload_to_s3(aws_s3_bucket_upload, f"stig-results/{dir_name}/{file}", account, image_digest, image)
        done = True



import click
import argparse
import os
import subprocess
import signal

import itertools
import threading
import time
import sys

from .general import sync_policies, sync_profiles_from_tar, check_cinc_installed, check_saf_installed
from .static import static_analysis
from .vm import run_stig_over_ssh
from .runtime import runtime_analysis, saf_convert_output
from .inputs import collect_inputs, runtime_get_image_digest, get_runtime_cluster
from .provision import install_cinc, install_train_plugin
from .generate import generate_input_file
from .review import validate_file, create_output_rollup
from anchorestig import __version__

@click.group()
def main():
    pass


@click.command()
@click.option('--username', '-u', help='Username for private registry')
@click.option('--password', '-p', help="Password for private registry")
@click.option('--url', '-r', help="URL for private registry")
@click.option('--aws-bucket', '-b', help="S3 upload. Specify bucket name")
@click.option('--account', '-a', help="Anchore STIG UI account. Required for S3 upload")
@click.option('--insecure', '-s', is_flag=True, default=False, help="Allow insecure registries or registries with custom certs")
@click.option('--profile', '-l', default="auto", help="Specify profile to run. Can be the name of an existing profile or the path to a custom profile")
@click.option('--input-file', '-i', help="Specify the path to a custom input file to run with a profile.")
@click.option('--sync', '-y', is_flag=True, default=False, help="Sync policies from Anchore")
@click.option('--sync-from-file', '-t', help="Sync policies from tar file provided by Anchore. Provide the path to the tar file.")
@click.argument('image')
def static(username, password, url, insecure, image, aws_bucket, account, profile, input_file, sync, sync_from_file):
    """Run static analysis"""
    if sync:
        sync_policies()
        print("Policies successfully downloaded.")
    if sync_from_file:
        sync_profiles_from_tar(sync_from_file)
        print("Policies successfully updated.")
    check_cinc_installed()
    check_saf_installed()
    if not input_file:
        input_file = "default"
    done = False
    def animate():
        print('Running STIG')
        for c in itertools.cycle(['|', '/', '-', '\\']):
            if done:
                break
            sys.stdout.write('\r' + c)
            sys.stdout.flush
            time.sleep(0.1)
        sys.stdout.write('\rDone!     ')
    t = threading.Thread(target=animate)
    t.start()
    aws = aws_bucket
    static_analysis(username, password, url, insecure, image, aws, account, profile, input_file)
    done = True

@click.command()
@click.option("--image", "-i", help="Specify profile to use. Available options are ubuntu-20.04, ubuntu-22.04, ubi8, ubi9, postgres9, apache-tomcat9, crunchy-postgresql, jboss, jre7, mongodb, nginx")
@click.option("--pod", "-p", help="Any running pod running an image that runs one of the specififed profile's software")
@click.option("--container", "-c", help="Container in the pod to run against")
@click.option("--outfile", "-o", help="Output file name. Only JSON output filetype is supported (include the '.json' extension with the output file name in CLI)")
@click.option("--namespace", "-n", help="Namespace the pod is located in")
@click.option("--usecontext", "-u", help="Specify the kubernetes context to use")
@click.option("--aws-bucket", "-b", help="Specify the S3 bucket to upload results to. Omit to skip upload")
@click.option("--account", "-a", help="Specify the Anchore STIG UI account to associate the S3 upload with. Omit to skip upload")
@click.option('--interactive', '-t', is_flag=True, default=False, help="Run in interactive mode")
@click.option('--input-file', '-f', help="Specify the path to a custom input file to run with a profile.")
@click.option('--sync', '-s', is_flag=True, default=False, help="Sync policies from Anchore")
@click.option('--sync-from-file', '-y', help="Sync policies from tar file provided by Anchore. Provide the path to the tar file.")
def runtime(image, pod, container, outfile, namespace, usecontext, aws_bucket, account, interactive, input_file, sync, sync_from_file):
    """Run runtime analysis"""
    print("Runtime Analysis")
    aws = aws_bucket
    if not aws:
        aws = "skip_upload"
    if not account:
        account = "skip_upload"
    if sync:
        sync_policies()
        print("Policies successfully downloaded.")
        if not interactive or not pod or not container:
            return
    if sync_from_file:
        sync_profiles_from_tar(sync_from_file)
        print("Policies successfully updated.")
        if not interactive or not pod or not container:
            return
    check_saf_installed()
    if interactive == True:
        input_image, input_pod, input_container, input_namespace, input_usecontext, input_cluster, input_outfile, input_aws_s3_bucket_upload, input_account, input_image_digest, input_image_name = collect_inputs()
        input_outfile = f"{input_outfile.rsplit('.', 1)[0]}/{input_outfile}"
        runtime_analysis(input_image, input_pod, input_container, input_namespace, input_usecontext, input_cluster, input_outfile, input_aws_s3_bucket_upload, input_account, input_image_digest, input_image_name)
    else:
        input_image_digest, input_image_name = runtime_get_image_digest(pod, namespace, container)
        input_cluster = get_runtime_cluster(usecontext)
        input_image, input_pod, input_container, input_namespace, input_usecontext, input_outfile, input_aws_s3_bucket_upload, input_account = image, pod, container, namespace, usecontext, outfile, aws, account
        input_outfile = f"{input_outfile.rsplit('.', 1)[0]}/{input_outfile}"
        runtime_analysis(input_image, input_pod, input_container, input_namespace, input_usecontext, input_cluster, input_outfile, input_aws_s3_bucket_upload, input_account, input_image_digest, input_image_name)
    saf_convert_output(input_outfile)

@click.command()
@click.option('--user', '-u', help='Username for SSH Host', required=True)
@click.option('--password', '-p', default="usekey", help="Password for SSH Host")
@click.option('--key', '-k', default="nokey", help="PEM Key path for SSH Host")
@click.option('--host', '-h', help='Username for SSH Host', required=True)
@click.option('--profile', '-l', default="indeterminate", help="Specify profile to run. Can be the name of an existing profile or the path to a custom profile. Existing profile are Available options are ubuntu2004, ubuntu2204, ubi8, ubi9, postgres9, apache-tomcat9, crunchy-postgresql, jboss, jre7, mongodb, nginx", required=True)
@click.option('--input-file', '-i', help="Specify the path to a custom input file to run with a profile.")
def vm(user, password, key, host, profile, input_file):
    """Run vm analysis"""
    if not input_file:
        input_file = "default"
    done = False
    def animate():
        print('Running STIG')
        for c in itertools.cycle(['|', '/', '-', '\\']):
            if done:
                break
            sys.stdout.write('\r' + c)
            sys.stdout.flush
            time.sleep(0.1)
        sys.stdout.write('\rDone!     ')
    t = threading.Thread(target=animate)
    t.start()
    run_stig_over_ssh(profile, input_file, host, user, password, key)
    done = True

@click.command()
@click.option('--install', '-i', is_flag=True, default=False, help="Install the necessary version of CINC")
@click.option("--privileged", "-s", is_flag=True, default=False, help="Install CINC with sudo.")
@click.option("--plugin", "-p", is_flag=True, default=False, help="Install the CINC Train K8S Plugin")
def provision(install, privileged, plugin):
    """Install required tools - Please note this tool is experimental. Refer to documentation for instructions about installing required tooling."""
    if install:
        install_cinc(privileged)
    if plugin:
        install_train_plugin()

@click.command()
@click.argument('profile_name', required=False)
def generate(profile_name):
    """Generate an example inputs file. Note: the generated file will be the default file used if no input file is specified."""
    profile_list = """
    Profile name required. Please specify one of the following as an argument.
    Available profile names:
    apache-tomcat9
    crunchy-postgresql
    jboss
    jre7
    mongodb
    nginx
    postgres9
    ubi8
    ubi9
    ubuntu2004
    ubuntu2204
    """
    available_profiles = ["apache-tomcat9", "crunchy-postgresql", "jboss", "jre7", "mongodb", "nginx", "postgres9", "ubi8", "ubi9", "ubuntu2004", "ubuntu2204"]
    if not profile_name:
        print(profile_list)
    elif profile_name not in available_profiles:
        print(profile_list)
    else:
        generate_input_file(profile_name)

@click.command()
@click.argument('output_file_path', required=True)
def review(output_file_path):
    """Generates an in-terminal rollup of a STIG result file"""
    valid_file = validate_file(output_file_path)
    if not valid_file:
        print("Input file is not valid, please try again.")
        exit()
    else:
        create_output_rollup(output_file_path)

@click.command()
def version():
    """Print the current version of Anchore STIG"""
    print(__version__)

main.add_command(static)
main.add_command(runtime)
main.add_command(provision)
main.add_command(vm)
main.add_command(generate)
main.add_command(version)
main.add_command(review)

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name='anchorestig',
    version='0.75.3',
    description='A tool for performing static and runtime analysis using STIGs',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Anchore Inc.',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'anchorestig=anchorestig.main:main',
        ],
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'click',
        'boto3',
    ],
    python_requires='>=3.6',
)
